use clap::{Parser, Subcommand};
use sqlx::{Acquire, PgPool};

use crate::model::{
    member::{Groups, Roles},
    Member,
};

#[derive(Parser)]
#[command(version, about, long_about = None)]
struct Cli {
    #[command(subcommand)]
    command: Option<Commands>,
}

#[derive(Subcommand)]
enum Commands {
    Serve,
    CreateAdminMember,
    SetMemberAdmin { relatiecode: String },
}

pub async fn parse(pool: PgPool) {
    let cli = Cli::parse();

    match &cli.command {
        Some(Commands::Serve) => {
            crate::util::serve::serve(pool).await;
        }
        Some(Commands::CreateAdminMember) => {
            create_admin_member(&pool).await.unwrap();
        }
        Some(Commands::SetMemberAdmin {
            relatiecode: member_id,
        }) => {
            set_member_admin(&pool, member_id).await.unwrap();
        }
        None => {}
    }
}

pub async fn create_admin_member(pool: &PgPool) -> Result<(), sqlx::Error> {
    use crate::database::model::Member as DbMember;

    let member = DbMember {
        member_id: "D000000".to_string(),
        first_name: "Admin".to_string(),
        full_name: "Admin Admin".to_string(),
        registration_token: None,
        diploma: None,
        groups: Groups::empty(),
        roles: Roles::ADMIN,
    };

    let mut transaction = pool.begin().await?;

    DbMember::insert_many(&mut transaction, vec![member]).await?;

    transaction.commit().await?;

    Ok(())
}

pub async fn set_member_admin(pool: &PgPool, id: &str) -> Result<(), sqlx::Error> {
    sqlx::query!(
        "
            UPDATE ONLY members SET roles = $1 WHERE member_id = $2
        ",
        Roles::ADMIN.bits() as i64,
        id
    )
    .execute(pool)
    .await?;

    Ok(())
}
